import * as createError from 'http-errors';
import * as express from 'express';
import * as path from 'path';
import * as cookieParser from 'cookie-parser';
import * as logger from 'morgan';
import * as passport from 'passport';
import * as helmet from 'helmet';

require('./components/authentication/passportSetup'); // runs some passport initialization
import { sessionsSetup } from './components/sessions/sessions';

var indexRouter = require('./routes/index');
var usersRouter = require('./components/users/usersRoutes');
// SYSTEM-BUILDER-router.require


const debug = require('debug')('frame:app-root');
var app = express();

sessionsSetup(app);
app.use(helmet());
app.use(passport.initialize());
app.use(passport.session());

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');

app.use(logger('dev'));
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

app.use('/', indexRouter);
app.use('/api/v1/users', usersRouter);
// SYSTEM-BUILDER-app.use


// catch 404 and forward to error handler
app.use(function(req, res, next) {
  next(createError(404));
});

// error handler
app.use(function(err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get('env') === 'development' ? err : {};

  // render the error page
  res.status(err.status || 500);
  res.render('error');
});

module.exports = app;
