import * as session from 'express-session';
import * as mySQLSession from 'express-mysql-session';

const debug = require('debug')('frame:session');

function sessionsSetup(app) {
  // setup sessions
  let MySQLStore = mySQLSession(session);
  const DOMAIN = process.env.DOMAIN || 'localhost';
  debug(`Domain: ${DOMAIN}`);
  let mysqlSessionOptions = {
    host: process.env.DB_HOST,
    port: process.env.DB_PORT || 3306,
    user: process.env.SESSION_DB_USER,
    password: process.env.SESSION_DB_PASSWORD,
    database: process.env.SESSION_DB_NAME
  };
  let sessionStore = new MySQLStore(mysqlSessionOptions);
  let sess = {
    secret: process.env.SESSION_SECRET, // can be an array. Use an array when rolling this key
    cookie: {
      secure: false,
      domain: DOMAIN
    },
    resave: false,
    store: sessionStore,
    saveUninitialized: false
  };

  debug(`Environment: ${app.get('env')}`);
  if (app.get('env') === 'production') {
    app.set('trust proxy', 1) // trust first proxy (usually nginx or loadbalancer)
    // sess.cookie.secure = true // serve secure cookies
    sess.cookie.secure = false // serve insecure cookies
  }

  app.use(session(sess));
}

export {
  sessionsSetup
}
