import { db } from '../../databases';
import { UsersMapper, DBUser } from './usersMapper';


class UserService {
  private userMapper = new UsersMapper(db);

  public initialize() {
    return Promise.resolve();
  }

  public create(profile: DBUser) {
    return this.userMapper.createUser(profile.sub, profile.email, profile.accessToken, profile.refreshToken);
  }

  public find(sub: string): Promise<DBUser> {
    return this.userMapper.findUserBySub(sub).then((matchingUsers) => {
      if (matchingUsers.length) {
        return matchingUsers[0];
      } else {
        throw new Error('User Not Found');
      }
    });
  }

  public update(profile: DBUser) {
    return this.userMapper.updateUser(profile.sub, profile.email, profile.accessToken, profile.refreshToken);
  }

  public findOrCreate(profile: DBUser) {
    return this.find(profile.sub).then((matchingUser: DBUser) => {
      return this.update(profile);
    }).catch((err) => {
      return this.create(profile);
    });
  }

  // SYSTEM-BUILDER-userService
}

const userService = new UserService();

export {
  userService
}
