# Altitude

Altitude is a custom migration tool for updating a database with migrations. The only dependency is the MySQL library. Migrations are simple. Just create jobs for Altitude to do.

```Typescript
interface Job {
  preCondition?: string;
  preConditionFieldName?: stirng;
  postCondition?: string;
  postConditionFieldName?: stirng;
  rollback?: string;
  name: string;
  query: string;
}
```

Then add the job to Altitude and tell it to run the migration.

```Typescript
let altitude = new Altitude();

let job: Job = {
  name: 'test',
  query: 'CREATE TABLE test_table (test VARCHAR(20));'
};

altitude.addMigration(job);

altitude.runMigrationByName('test');
```

All options are optional except for `name` and `query`. `query` can have multiple queries in the string as long as each are separated with a semicolon `;`. `name` must be unique otherwise only the first migration with that name will ever be run.

```Typescript
// optional properties

preCondition: `SELECT @@version NOT LIKE '5.0%' AND @@version NOT LIKE '5.1%' AS precondition;`, // query ran before the migration to check the current state of the database.
preConditionFieldName: 'precondition', // if the field in the precondition that needs to be checked isn't named precondition you can specify the correct field name here.
postCondition: `SELECT COUNT(*) as postcondition FROM StormTest.Users;` // same thing as precondition except run after the migration
postConditionFieldName: string; // same thing as the preConditionFieldName except for the postCondition
rollback: `DROP TABLE test_table;` // query that is run if the post condition fails.
```
