import { SystemDef } from './systemGenService';

let def: SystemDef = {
  name: 'Todo',
  storage: {
    tables: [
      {
        name: "task",
        relations: [
          {
            type: 'belongs-to',
            table: 'list',
          }
        ],
        columns: [
          {
            name: "name",
            type: "string",
            nullable: false,
          },
          {
            name: "description",
            type: "string",
            nullable: true,
          },
          {
            name: "completed",
            type: "boolean",
            nullable: false,
            default: 'false'
          },
          {
            name: "completed_date",
            type: "date",
            nullable: true,
          },
          {
            name: "metadata",
            type: "blob",
            nullable: true,
          }
        ]
      },
      {
        name: "list",
        relations: [],
        columns: [
          {
            name: "name",
            type: "string",
            unique: false,
            nullable: false,
          },
        ]
      },
      {
        name: "user",
        relations: [],
        columns: [
          {
            name: "name",
            type: "string",
            nullable: false,
          },
          {
            name: "password",
            type: "string",
            nullable: false,
          }
        ]
      }
    ],
    relations: [
      {
        left: 'list',
        relation: 'many-to-many',
        right: 'user',
        columns: [
          {
            name: 'access',
            type: 'string',
            unique: false,
            nullable: false,
          }
        ]
      }
    ]
  },
  components: [
    {
      component: 'task',
      endpoints: [
        {
          component: 'task',
          table: 'task',
          type: 'list',
          columns: [
            {
              name: 'name',
              table: 'task'
            },
            {
              name: 'description',
              table: 'task'
            },
            {
              name: 'completed',
              table: 'task'
            },
            {
              name: 'completed_date',
              table: 'task'
            },
          ],
          orderBy: [
            {
              column: {
                name: 'modified',
                table: 'task'
              },
              direction: 'desc'
            }
          ],
          filters: [
            {
              param: 'list_id',
              type: 'string',
              column: {
                name: 'list_id',
                table: 'task'
              },
              comparison: '=',
              required: true
            },
            {
              param: 'completed',
              type: 'boolean',
              column: {
                name: 'completed',
                table: 'task'
              },
              comparison: '=',
              required: false
            },
          ]
        },
        {
          component: 'task',
          table: 'task',
          type: 'search',
          columns: [
            {
              name: 'name',
              table: 'task'
            },
            {
              name: 'description',
              table: 'task'
            },
            {
              name: 'completed',
              table: 'task'
            },
            {
              name: 'completed_date',
              table: 'task'
            },
          ],
          orderBy: [
            {
              column: {
                name: 'modified',
                table: 'task'
              },
              direction: 'desc'
            }
          ],
          filters: [
            {
              param: 'list_id',
              type: 'string',
              column: {
                name: 'list_id',
                table: 'task'
              },
              comparison: '=',
              required: true
            },
            {
              param: 'completed',
              type: 'boolean',
              column: {
                name: 'completed',
                table: 'task'
              },
              comparison: '=',
              required: false
            },
            {
              param: 'name',
              type: 'string',
              column: {
                name: 'name',
                table: 'task'
              },
              comparison: 'LIKE',
              required: false
            },
            {
              param: 'description',
              type: 'string',
              column: {
                name: 'description',
                table: 'task'
              },
              comparison: 'LIKE',
              required: false
            },
          ]
        },
        {
          component: 'task',
          table: 'task',
          type: 'count',
          columns: [
            {
              name: 'name',
              table: 'task'
            },
            {
              name: 'description',
              table: 'task'
            },
            {
              name: 'completed',
              table: 'task'
            },
            {
              name: 'completed_date',
              table: 'task'
            },
          ],
          orderBy: [
            {
              column: {
                name: 'modified',
                table: 'task'
              },
              direction: 'desc'
            }
          ],
          filters: [
            {
              param: 'list_id',
              type: 'string',
              column: {
                name: 'list_id',
                table: 'task'
              },
              comparison: '=',
              required: true
            },
            {
              param: 'completed',
              type: 'boolean',
              column: {
                name: 'completed',
                table: 'task'
              },
              comparison: '=',
              required: false
            },
          ]
        },
        {
          component: 'task',
          table: 'task',
          type: 'create',
          columns: [
            {
              name: 'name',
              table: 'task',
              param: 'name',
              type: 'string',
              required: true
            },
            {
              name: 'list_id',
              table: 'task',
              param: 'list',
              type: 'string',
              required: true
            },
            {
              name: 'description',
              table: 'task',
              param: 'description',
              type: 'string'
            },
            {
              name: 'completed',
              table: 'task',
              param: 'completed',
              type: 'boolean',
            }
          ],
        },
        {
          component: 'task',
          table: 'task',
          type: 'update',
          columns: [
            {
              name: 'name',
              table: 'task',
              param: 'name',
              type: 'string'
            },
            {
              name: 'description',
              table: 'task',
              param: 'description',
              type: 'string'
            },
            {
              name: 'completed',
              table: 'task',
              param: 'completed',
              type: 'boolean'
            },
            {
              name: 'completed_date',
              table: 'task',
              param: 'completedDate',
              type: 'string'
            },
          ],
          filters: [
            {
              column: {
                name: 'task_id',
                table: 'task'
              },
              comparison: '=',
              param: 'task_id',
              type: 'string',
              required: true
            },
          ]
        },
        {
          component: 'task',
          table: 'task',
          type: 'delete',
          columns:[],
          filters: [
            {
              column: {
                name: 'task_id',
                table: 'task'
              },
              comparison: '=',
              param: 'task_id',
              type: 'string',
              required: true
            },
          ]
        },
        {
          component: 'task',
          table: 'task',
          type: 'item',
          columns: [
            {
              name: 'task_id',
              table: 'task'
            },
            {
              name: 'name',
              table: 'task'
            },
            {
              name: 'description',
              table: 'task'
            },
            {
              name: 'completed',
              table: 'task'
            },
            {
              name: 'completed_date',
              table: 'task'
            },
          ],
          filters: [
            {
              column: {
                name: 'task_id',
                table: 'task'
              },
              comparison: '=',
              param: 'task_id',
              type: 'string',
              required: true
            },
          ]
        },
      ]
    },
    {
      component: 'list',
      endpoints: [
        {
          component: 'list',
          table: 'list',
          type: 'list',
          columns: [
            {
              name: 'name',
              table: 'list'
            },
          ],
          join: [
            {
              table: 'list_user',
              on: {
                left: {
                  name: 'list_id',
                  table: 'list'
                },
                right: {
                  name: 'list_id',
                  table: 'list_user'
                }
              }
            }
          ],
          orderBy: [
            {
              column: {
                name: 'modified',
                table: 'list'
              },
              direction: 'desc'
            }
          ],
          filters: [
            {
              param: 'user_id',
              type: 'string',
              column: {
                name: 'user_id',
                table: 'list_user'
              },
              comparison: '=',
              required: true
            },
          ]
        },
      ]
    }
  ],
};

export default def;
