import {ColumnDef, ColumnRef, ComponentDef, EndpointDef, SystemDef, TableDef} from "../systemGenService";
import path from "path";
import fs from "fs";
import {createDetailsInitValues} from "../helpers";

export function buildDetailsView(component: ComponentDef, outDir: string, systemDef: SystemDef): Promise<void> {
    let detailsEndpoints: EndpointDef[] = component.endpoints.filter((epd: EndpointDef) => {
        return epd.type === 'item';
    });
    if (detailsEndpoints.length) {
        return insertFEDetailsMarkup(detailsEndpoints[0], outDir).then(() => {
            return insertFEDetailsCode(detailsEndpoints[0], outDir, systemDef);
        });
    } else {
        return Promise.resolve();
    }
}

function insertFEDetailsCode(view: EndpointDef, outDir: string, systemDef: SystemDef) {
    return new Promise<void>((resolve) => {
        const separator: string = `// SYSTEM-BUILDER-init-props`;
        let fileLocation = path.join(outDir, `${view.component}Details.vue`)
        let initServiceFile: string = fs.readFileSync(fileLocation, 'utf8');
        let parts = initServiceFile.split(separator);
        parts[0] = parts[0] + createDetailsInitValues(view, systemDef);
        let newServiceFile = parts.join(separator);
        fs.writeFileSync(fileLocation, newServiceFile, 'utf8');
        resolve();
    });
}

function insertFEDetailsMarkup(view: EndpointDef, outDir: string) {
    return new Promise<void>((resolve) => {
        const separator: string = `<!-- SYSTEM-BUILDER-property-row -->`;
        let fileLocation = path.join(outDir, `${view.component}Details.vue`)
        let initServiceFile: string = fs.readFileSync(fileLocation, 'utf8');
        let parts = initServiceFile.split(separator);
        parts[0] = parts[0] + createDetailsMarkup(view);
        let newServiceFile = parts.join(separator);
        fs.writeFileSync(fileLocation, newServiceFile, 'utf8');
        resolve();
    });
}

function createDetailsMarkup(view: EndpointDef): string {
    let out = ``;

    view.columns.forEach((column: ColumnRef) => {
       out = out + `<div class="property-row">
      <label>${column.name}: </label>
      <div>{{ ${view.component}.${column.name} }}</div>
    </div>

    `;
    });

    return out;
}



