import {ColumnDef, ColumnRef, ComponentDef, EndpointDef, SystemDef, TableDef} from "../systemGenService";
import path from "path";
import fs from "fs";
import {createDetailsInitValues, uppercaseFirstLetter} from "../helpers";

export function buildEditorView(component: ComponentDef, outDir: string, systemDef: SystemDef): Promise<void> {
    let detailsEndpoints: EndpointDef[] = component.endpoints.filter((epd: EndpointDef) => {
        return epd.type === 'item';
    });
    if (detailsEndpoints.length) {
        return insertFEDetailsMarkup(detailsEndpoints[0], outDir).then(() => {
            return insertFEDetailsCode(detailsEndpoints[0], outDir, systemDef);
        });
    } else {
        return Promise.resolve();
    }
}

function insertFEDetailsCode(view: EndpointDef, outDir: string, systemDef: SystemDef) {
    return new Promise<void>((resolve) => {
        const separator: string = `// SYSTEM-BUILDER-init-props`;
        let fileLocation = path.join(outDir, `${view.component}Editor.vue`)
        let initServiceFile: string = fs.readFileSync(fileLocation, 'utf8');
        let parts = initServiceFile.split(separator);
        parts[0] = parts[0] + createDetailsInitValues(view, systemDef);
        let newServiceFile = parts.join(separator);
        fs.writeFileSync(fileLocation, newServiceFile, 'utf8');
        resolve();
    });
}

function insertFEDetailsMarkup(view: EndpointDef, outDir: string) {
    return new Promise<void>((resolve) => {
        const separator: string = `<!-- SYSTEM-BUILDER-input-row -->`;
        let fileLocation = path.join(outDir, `${view.component}Editor.vue`)
        let initServiceFile: string = fs.readFileSync(fileLocation, 'utf8');
        let parts = initServiceFile.split(separator);
        parts[0] = parts[0] + createDetailsMarkup(view);
        let newServiceFile = parts.join(separator);
        fs.writeFileSync(fileLocation, newServiceFile, 'utf8');
        resolve();
    });
}

function createDetailsMarkup(view: EndpointDef): string {
    let out = ``;
    let inputType: {[key: string]: string} = {
        "blob": "text",
        "boolean": "checkbox",
        "date": "date",
        "dateTIME": "datetime-local",
        "number": "number",
        "string": "text",
    };

    view.columns.forEach((column: ColumnRef) => {
        let inputTypeValue = 'text';
        if (column.type) {
            inputTypeValue = inputType[column.type];
        }
       out = out + `<div class="input-row">
      <label>${uppercaseFirstLetter(view.component)} ${uppercaseFirstLetter(column.name)}: </label>
      <input class="form-control" type="${inputTypeValue}" placeholder="${column.name}" v-model="${view.component}.${column.name}" />
    </div>

    `;
    });

    return out;
}



