import {ComponentDef, EndpointDef, SystemDef} from "../systemGenService";
import path from "path";
import fs from "fs";

export function buildRoute(component: ComponentDef, outDir: string, systemDef: SystemDef): Promise<void> {
    let listEndpoints: EndpointDef[] = component.endpoints.filter((epd: EndpointDef) => {
        return epd.type === 'list';
    });
    if (listEndpoints.length) {
        return new Promise<void>((resolve) => {
            const separator: string = `// SYSTE-BUILDER-routes`;
            let fileLocation = path.join(process.cwd(), `frontend-${systemDef.name}`, 'src', 'router.ts');
            let initServiceFile: string = fs.readFileSync(fileLocation, 'utf8');
            let parts = initServiceFile.split(separator);
            parts[0] = parts[0] + createRouteCode(component, systemDef);
            let newServiceFile = parts.join(separator);
            fs.writeFileSync(fileLocation, newServiceFile, 'utf8');
            resolve();
        });
    }
    return Promise.resolve();
}

function createRouteCode(component: ComponentDef, systemDef: SystemDef): string {
    let out = `{
            path: '/${component.component}',
            name: '${component.component}',
            component: () => import(/* webpackChunkName: "${component.component}" */ './components/${component.component}/${component.component}List.vue')
        },
        `;

    return out;
}